/*
 * Copyright (c) 2024 SUSE LLC
 *
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail, you may
 * find current contact information at www.novell.com.
 */


#include <iostream>

#include <snapper/AppUtil.h>

#include "../utils/text.h"

#include "BackupConfig.h"
#include "GlobalOptions.h"
#include "TheBigThing.h"
#include "cmd.h"


namespace snapper
{

    using namespace std;


    void
    help_cleanup()
    {
	cout << "  " << _("Cleanup:") << '\n'
	     << "\t" << _("snbk cleanup") << '\n'
	     << '\n';
    }


    void
    command_cleanup(GlobalOptions& global_options, GetOpts& get_opts, Hahas& hahas,
		    ProxySnappers* snappers)
    {
	ParsedOpts opts = get_opts.parse("cleanup", GetOpts::no_options);

	if (get_opts.has_args())
	{
	    SN_THROW(OptionsException(_("Command 'cleanup' does not take arguments.")));
	}

	for (const Haha& haha : hahas)
	{
	    if (!global_options.quiet())
		cout << sformat(_("Running cleanup for backup config '%s'."), haha.name.c_str()) << endl;

	    TheBigThings the_big_things(haha.backup_config, snappers, global_options.verbose());

	    the_big_things.cleanup(haha.backup_config);
	}
    }

}
